@@ -10,12 +10,91 @@ from nilearn import plotting
 from tqdm import trange
 
 from osl_dynamics import array_ops
-from osl_dynamics.analysis import gmm
+from osl_dynamics.analysis import gmm, static
 from osl_dynamics.analysis.spectral import get_frequency_args_range
 from osl_dynamics.utils.parcellation import Parcellation
 from osl_dynamics.utils.misc import override_dict_defaults
 
 
+def sliding_window_connectivity(
+    data,
+    window_length,
+    step_size=None,
+    conn_type="corr",
+    concatenate=False,
+):
+    """Calculate sliding window connectivity.
+
+    Parameters
+    ----------
+    data : list or np.ndarray
+        Time series data. Shape must be (n_subjects, n_samples, n_channels)
+        or (n_samples, n_channels).
+    window_length : int
+        Window length in samples.
+    step_size : int
+        Number of samples to slide the window along the time series.
+        If None is passed, then a 50% overlap is used.
+    conn_type : str
+        Metric to use to calculate pairwise connectivity in the network.
+        Should "corr" for Pearson correlation or "cov" for covariance.
+    concatenate : bool
+        Should we concatenate the sliding window connectivities from each subject
+        into one big time series?
+
+    Returns
+    -------
+    sliding_window_conn : list or np.ndarray
+        Time series of connectivity matrices. Shape is (n_subjects, n_windows,
+        n_channels, n_channels) or (n_windows, n_channels, n_channels).
+    """
+    # Validation
+    if conn_type not in ["corr", "cov"]:
+        raise ValueError("conn_type must be 'corr' or 'cov'.")
+
+    if conn_type == "cov":
+        metric = np.cov
+    else:
+        metric = np.corrcoef
+
+    if step_size is None:
+        step_size = window_length // 2
+
+    if isinstance(data, np.ndarray):
+        if data.ndim != 3:
+            data = [data]
+
+    # Calculate sliding window connectivity for each subject
+    sliding_window_conn = []
+    for i in trange(len(data), desc="Calculating connectivity", ncols=98):
+        n_samples = data[i].shape[0]
+        n_channels = data[i].shape[1]
+
+        # Define indices of time points that start windows
+        time_idx = range(0, n_samples, step_size)
+        n_windows = n_samples // step_size
+
+        # Trim the data to only include complete window
+        data[i] = data[i][: n_windows * window_length]
+
+        # Preallocate an array to hold moving average values
+        swc = np.empty([n_windows, n_channels, n_channels], dtype=np.float32)
+
+        # Compute connectivity matrix for each window
+        for k in range(n_windows):
+            j = time_idx[k]
+            window = data[i][j : j + window_length]
+            swc[k] = metric(window, rowvar=False)
+
+        # Add to list to return
+        sliding_window_conn.append(swc)
+
+    if concatenate or len(sliding_window_conn) == 1:
+        sliding_window_conn = sliding_window_conn[0]
+
+    return sliding_window_conn
+
+
 def covariance_from_spectra(
     frequencies,
     power_spectra,
@@ -30,9 +109,9 @@ def covariance_from_spectra(
         Frequency axis of the PSDs. Only used if frequency_range is given.
     power_spectra : np.ndarray
         Power/cross spectra for each channel. Shape is (n_modes, n_channels,
-        n_channels, n_f).
+        n_channels, n_freq).
     components : np.ndarray
-        Spectral components. Shape is (n_components, n_f).
+        Spectral components. Shape is (n_components, n_freq).
     frequency_range : list
         Frequency range to integrate the PSD over (Hz). Default is full range.
 
@@ -45,9 +124,9 @@ def covariance_from_spectra(
 
     # Validation
     error_message = (
-        "A (n_channels, n_channels, n_frequency_bins), "
-        + "(n_modes, n_channels, n_channels, n_frequency_bins) or "
-        + "(n_subjects, n_modes, n_channels, n_channels, n_frequency_bins) "
+        "A (n_channels, n_channels, n_freq), "
+        + "(n_modes, n_channels, n_channels, n_freq) or "
+        + "(n_subjects, n_modes, n_channels, n_channels, n_freq) "
         + "array must be passed."
     )
     power_spectra = array_ops.validate(
@@ -68,7 +147,7 @@ def covariance_from_spectra(
         )
 
     # Dimensions
-    n_subjects, n_modes, n_channels, n_channels, n_f = power_spectra.shape
+    n_subjects, n_modes, n_channels, n_channels, n_freq = power_spectra.shape
     if components is None:
         n_components = 1
     else:
@@ -78,7 +157,7 @@ def covariance_from_spectra(
     covar = []
     for i in range(n_subjects):
         # Cross spectral densities
-        csd = power_spectra[i].reshape(-1, n_f)
+        csd = power_spectra[i].reshape(-1, n_freq)
         csd = abs(csd)
 
         if components is not None:
@@ -115,9 +194,9 @@ def mean_coherence_from_spectra(
         Frequency axis of the PSDs. Only used if frequency_range is given.
     coherence : np.ndarray
         Coherence for each channel. Shape is (n_modes, n_channels,
-        n_channels, n_f).
+        n_channels, n_freq).
     components : np.ndarray
-        Spectral components. Shape is (n_components, n_f).
+        Spectral components. Shape is (n_components, n_freq).
     frequency_range : list
         Frequency range to integrate the PSD over (Hz).
 
@@ -131,9 +210,9 @@ def mean_coherence_from_spectra(
 
     # Validation
     error_message = (
-        "a 3D numpy array (n_channels, n_channels, n_frequency_bins) "
+        "a 3D numpy array (n_channels, n_channels, n_freq) "
         + "or 4D numpy array (n_modes, n_channels, n_channels, "
-        + "n_frequency_bins) must be passed for spectra."
+        + "n_freq) must be passed for spectra."
     )
     coherence = array_ops.validate(
         coherence,
@@ -153,7 +232,7 @@ def mean_coherence_from_spectra(
         )
 
     # Dimensions
-    n_subjects, n_modes, n_channels, n_channels, n_f = coherence.shape
+    n_subjects, n_modes, n_channels, n_channels, n_freq = coherence.shape
     if components is None:
         n_components = 1
     else:
@@ -164,7 +243,7 @@ def mean_coherence_from_spectra(
     for i in range(n_subjects):
 
         # Concatenate over modes
-        coh = coherence[i].reshape(-1, n_f)
+        coh = coherence[i].reshape(-1, n_freq)
 
         if components is not None:
             # Coherence for each spectral component
@@ -202,6 +281,51 @@ def mean_connections(conn_map):
     return np.mean(conn_map, axis=-1)
 
 
+def eigenvectors(conn_map, n_eigenvectors=1, absolute_value=False, as_network=False):
+    """Calculate eigenvectors of a connectivity matrix.
+
+    Parameters
+    ----------
+    conn_map : np.ndarray
+        Connectivity matrix. Shape must be (..., n_channels, n_channels).
+    n_eigenvectors : int
+        Number of eigenvectors to include.
+    absolute_value : bool
+        Should we take the absolute value of the connectivity matrix before
+        calculating the eigen decomposition?
+    as_network : bool
+        Should we return a matrix?
+
+    Returns
+    -------
+    eigenvectors : np.ndarray.
+        Eigenvectors. Shape is (n_eigenvectors, ..., n_channels, n_channels)
+        if as_network=True, otherwise it is (n_eigenvectors, ..., n_channels).
+        If n_eigenvectors=1, the first dimension is removed.
+    """
+    if absolute_value:
+        # Take absolute value
+        conn_map = abs(conn_map)
+
+    # Calculate eigen decomposition
+    _, eigenvectors = np.linalg.eigh(conn_map)
+
+    # Reorder from ascending eigenvalues to descending
+    eigenvectors = eigenvectors[..., ::-1]
+
+    # Keep the requested number of eigenvectors and make the first axis
+    # specify the eigenvector
+    eigenvectors = np.rollaxis(eigenvectors[..., :n_eigenvectors], -1)
+
+    if as_network:
+        # Calculate the outer product using the eigenvectors
+        eigenvectors = np.expand_dims(eigenvectors, axis=-1) @ np.expand_dims(
+            eigenvectors, axis=-2
+        )
+
+    return np.squeeze(eigenvectors)
+
+
 def gmm_threshold(
     conn_map,
     subtract_mean=False,
@@ -212,6 +336,7 @@ def gmm_threshold(
     one_component_percentile=0,
     n_sigma=0,
     sklearn_kwargs={},
+    show=False,
     filename=None,
     plot_kwargs={},
 ):
@@ -248,6 +373,8 @@ def gmm_threshold(
     sklearn_kwargs : dict
         Dictionary of keyword arguments to pass to
         sklearn.mixture.BayesianGaussianMixture().
+    show : bool
+        Should we show the GMM fit to the distribution of conn_map.
     filename : str
         Filename to save fit to.
     plot_kwargs : dict
@@ -269,6 +396,7 @@ def gmm_threshold(
         one_component_percentile,
         n_sigma,
         sklearn_kwargs,
+        show,
         filename,
         plot_kwargs,
     )
@@ -286,6 +414,7 @@ def fit_gmm(
     one_component_percentile=0,
     n_sigma=0,
     sklearn_kwargs={},
+    show=False,
     filename=None,
     plot_kwargs={},
 ):
@@ -295,7 +424,8 @@ def fit_gmm(
     Parameters
     ----------
     conn_map : np.ndarray
-        Connectivity map.
+        Connectivity map. Shape must be (n_components, n_modes, n_channels, n_channels)
+        or (n_modes, n_channels, n_channels) or (n_channels, n_channels).
     subtract_mean : bool
         Should we subtract the mean over modes before fitting a GMM?
     mean_weights: np.ndarray
@@ -321,6 +451,8 @@ def fit_gmm(
         Dictionary of keyword arguments to pass to
         sklearn.mixture.GaussianMixture(). Default is
         {"max_iter": 5000, "n_init": 10}.
+    show : bool
+        Should we show the GMM fit to the distribution of conn_map.
     filename : str
         Filename to save fit to.
     plot_kwargs : dict
@@ -396,6 +528,7 @@ def fit_gmm(
                 one_component_percentile=one_component_percentile,
                 n_sigma=n_sigma,
                 sklearn_kwargs=sklearn_kwargs,
+                show_plot=show,
                 plot_filename=plot_filename,
                 plot_kwargs=plot_kwargs,
                 print_message=False,
@@ -425,7 +558,8 @@ def threshold(
         (n_modes, n_channels, n_channels) or (n_channels, n_channels).
     percentile : float or np.ndarray
         Percentile to threshold with. Should be between 0 and 100.
-        Can be a numpy array of shape (n_modes,) or (n_components, n_modes).
+        Can be a numpy array of shape (n_components, n_modes), (n_modes,)
+        or a float.
     subtract_mean : bool
         Should we subtract the mean over modes before thresholding?
         The thresholding is only done to identify edges, the values returned in
@@ -462,12 +596,17 @@ def threshold(
     n_modes = conn_map.shape[1]
     n_channels = conn_map.shape[2]
 
+    # Validatation
     if isinstance(percentile, float) or isinstance(percentile, int):
         percentile = percentile * np.ones([n_components, n_modes])
 
-    if percentile.ndim == 1:
-        # A (n_modes,) array has been passed, add the n_components dimension
-        percentile = percentile[np.newaxis, ...]
+    percentile = array_ops.validate(
+        percentile,
+        correct_dimensionality=2,
+        allow_dimensions=[0, 1],
+        error_message="percentile must be of shape "
+        + "(n_components, n_modes), (n_modes,) or float",
+    )
 
     # Copy the original connectivity map
     c = conn_map.copy()
@@ -524,8 +663,8 @@ def separate_edges(conn_map):
 
 def save(
     connectivity_map,
-    filename,
     parcellation_file,
+    filename=None,
     component=None,
     threshold=0,
     glassbrain=False,
@@ -542,10 +681,11 @@ def save(
     connectivity_map : np.ndarray
         Matrices containing connectivity strengths to plot.
         Shape must be (n_modes, n_channels, n_channels) or (n_channels, n_channels).
-    filename : str
-        Output filename.
     parcellation_file : str
         Name of parcellation file used.
+    filename : str
+        Output filename.
+        Optional, if None is passed then the image is shown on screen.
     component : int
         Spectral component to save.
     threshold : float or np.ndarray
@@ -559,10 +699,11 @@ def save(
         Keyword arguments to pass to the nilearn plotting function.
     """
     # Validation
-    if glassbrain and Path(filename).suffix != ".html":
-        raise ValueError(
-            "If glassbrain=True then filename must have a .html extension."
-        )
+    if filename is not None:
+        if glassbrain and Path(filename).suffix != ".html":
+            raise ValueError(
+                "If glassbrain=True then filename must have a .html extension."
+            )
 
     error_message = (
         "Dimensionality of connectivity_map must be 3 or 4, "
@@ -601,9 +742,12 @@ def save(
         kwargs = override_dict_defaults(default_plot_kwargs, plot_kwargs)
 
         # Output filename
-        output_file = "{fn.parent}/{fn.stem}{i:0{w}d}{fn.suffix}".format(
-            fn=Path(filename), i=i, w=len(str(n_modes))
-        )
+        if filename is None:
+            output_file = None
+        else:
+            output_file = "{fn.parent}/{fn.stem}{i:0{w}d}{fn.suffix}".format(
+                fn=Path(filename), i=i, w=len(str(n_modes))
+            )
 
         if glassbrain:
             # The colour bar range is determined by the max value in the matrix
@@ -621,7 +765,10 @@ def save(
                 edge_threshold=f"{threshold[i] * 100}%",
                 **kwargs,
             )
-            connectome.save_as_html(output_file)
+            if filename is not None:
+                connectome.save_as_html(output_file)
+            else:
+                return connectome
 
         else:
             # If all connections are zero don't add a colourbar
@@ -637,3 +784,48 @@ def save(
                 output_file=output_file,
                 **kwargs,
             )
+
+
+def spectral_reordering(corr_mat):
+    """Spectral re-ordering for correlation matrices.
+
+    Parameters
+    ----------
+    corr_mat : np.ndarray
+        Correlation matrix.
+
+    Returns
+    -------
+    reorder_corr_mat : np.ndarray
+        Re-ordered correlation matrix.
+    order : np.ndarray
+        New ordering.
+    """
+    # Add one to make all entries postive
+    C = corr_mat + 1
+
+    # Compute Q
+    Q = -C
+    np.fill_diagonal(Q, 0)
+    Q -= np.sum(Q, axis=0)
+
+    # Compute t
+    t = np.diag(1.0 / np.sqrt(np.sum(C, axis=0)))
+
+    # Compute D
+    D = np.dot(np.dot(t, Q), t)
+
+    # Eigevalue decomposition
+    D, W = np.linalg.eig(D)
+    v = W[:, 1]
+
+    # Scale v
+    v = np.dot(t, v)
+
+    # Find permutations
+    order = np.argsort(v)
+
+    # Reorder
+    reorder_corr_mat = corr_mat[order, :][:, order]
+
+    return reorder_corr_mat, order
