@@ -21,8 +21,9 @@ from osl_dynamics.inference import initializers
 from osl_dynamics.models import dynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
 from osl_dynamics.inference.layers import (
-    MeanVectorsLayer,
+    VectorsLayer,
     CovarianceMatricesLayer,
+    DiagonalMatricesLayer,
     CategoricalLogLikelihoodLossLayer,
 )
 
@@ -46,15 +47,18 @@ class Config(BaseModelConfig):
     sequence_length : int
         Length of sequence passed to the inference network and generative model.
     learn_means : bool
-        Should we make the mean vectors for each mode trainable?
+        Should we make the mean vectors for each state trainable?
     learn_covariances : bool
-        Should we make the covariance matrix for each mode trainable?
+        Should we make the covariance matrix for each staet trainable?
     initial_means : np.ndarray
-        Initialisation for mean vectors.
+        Initialisation for state means.
     initial_covariances : np.ndarray
-        Initialisation for mode covariances.
+        Initialisation for state covariances. If diagonal_covariances=True
+        and full matrices are passed, the diagonal is extracted.
+    diagonal_covariances : bool
+        Should we learn diagonal covariances?
     covariances_epsilon : float
-        Error added to mode covariances for numerical stability.
+        Error added to state covariances for numerical stability.
     initial_trans_prob : np.ndarray
         Initialisation for trans prob matrix
     learn_trans_prob : bool
@@ -65,6 +69,20 @@ class Config(BaseModelConfig):
         Mini-batch size.
     learning_rate : float
         Learning rate.
+    trans_prob_update_delay : float
+        We update the transition probability matrix as
+        trans_prob = (1-rho) * trans_prob + rho * trans_prob_update,
+        where rho = (100 * epoch / n_epochs + 1 + trans_prob_update_delay)
+        ** -trans_prob_update_forget. This is the delay parameter.
+    trans_prob_update_forget : float
+        We update the transition probability matrix as
+        trans_prob = (1-rho) * trans_prob + rho * trans_prob_update,
+        where rho = (100 * epoch / n_epochs + 1 + trans_prob_update_delay)
+        ** -trans_prob_update_forget. This is the forget parameter.
+    observation_update_decay : float
+        Decay rate for the learning rate of the observation model.
+        We update the learning rate (lr) as
+        lr = config.learning_rate * exp(-observation_update_decay * epoch).
     n_epochs : int
         Number of training epochs.
     optimizer : str or tensorflow.keras.optimizers.Optimizer
@@ -82,14 +100,17 @@ class Config(BaseModelConfig):
     learn_covariances: bool = None
     initial_means: np.ndarray = None
     initial_covariances: np.ndarray = None
+    diagonal_covariances: bool = False
     covariances_epsilon: float = None
 
     initial_trans_prob: np.ndarray = None
     learn_trans_prob: bool = True
     state_probs_t0: np.ndarray = None
 
-    stochastic_update_delay: float = 5  # alpha
-    stochastic_update_forget: float = 0.7  # beta
+    # Learning rate schedule parameters
+    trans_prob_update_delay: float = 5  # alpha
+    trans_prob_update_forget: float = 0.7  # beta
+    observation_update_decay: float = 0.1
 
     def __post_init__(self):
         self.validate_observation_model_parameters()
@@ -125,11 +146,13 @@ class Model(ModelBase):
         self.set_trans_prob(self.config.initial_trans_prob)
         self.set_state_probs_t0(self.config.state_probs_t0)
 
-    def fit(self, dataset, epochs=None, lr_decay=0.1, take=1, **kwargs):
+    def fit(self, dataset, epochs=None, take=1, **kwargs):
         """Fit model to a dataset.
 
         Iterates between:
-        - Baum-Welch updates of latent variable time courses and transition probability matrix.
+
+        - Baum-Welch updates of latent variable time courses and transition
+          probability matrix.
         - TensorFlow updates of observation model parameters.
 
         Parameters
@@ -138,9 +161,6 @@ class Model(ModelBase):
             Training dataset.
         epochs : int
             Number of epochs.
-        lr_decay : float
-            We update the learning rate (lr) for the observation model as
-            lr = config.learning_rate * exp(-lr_decay * epoch).
         take : float
             Fraction of total batches to take.
         kwargs : keyword arguments
@@ -186,7 +206,9 @@ class Model(ModelBase):
             self._update_rho(n)
 
             # Set learning rate for the observation model
-            lr = self.config.learning_rate * np.exp(-lr_decay * n)
+            lr = self.config.learning_rate * np.exp(
+                -self.config.observation_update_decay * n
+            )
             backend.set_value(self.model.optimizer.lr, lr)
 
             # Loop over batches
@@ -526,8 +548,8 @@ class Model(ModelBase):
         # total number of iterations:
         # https://www.sciencedirect.com/science/article/pii/S1053811917305487
         self.rho = np.power(
-            100 * ind / self.config.n_epochs + 1 + self.config.stochastic_update_delay,
-            -self.config.stochastic_update_forget,
+            100 * ind / self.config.n_epochs + 1 + self.config.trans_prob_update_delay,
+            -self.config.trans_prob_update_forget,
         )
 
     def sample_state_time_course(self, n_samples):
@@ -558,34 +580,36 @@ class Model(ModelBase):
         return self.trans_prob
 
     def get_covariances(self):
-        """Get the covariances of each mode.
+        """Get the covariances of each state.
 
         Returns
         -------
         covariances : np.ndarray
             State covariances. Shape is (n_states, n_channels, n_channels).
         """
-        return dynemo_obs.get_covariances(self.model)
+        return dynemo_obs.get_covariances(self.model, self.config.diagonal_covariances)
 
     def get_means_covariances(self):
-        """Get the means and covariances of each mode.
+        """Get the means and covariances of each state.
 
         Returns
         -------
         means : np.ndarary
-            Mode means.
+            State means.
         covariances : np.ndarray
-            Mode covariances.
+            State covariances.
         """
-        return dynemo_obs.get_means_covariances(self.model)
+        return dynemo_obs.get_means_covariances(
+            self.model, self.config.diagonal_covariances
+        )
 
     def set_means(self, means, update_initializer=True):
-        """Set the means of each mode.
+        """Set the means of each state.
 
         Parameters
         ----------
         means : np.ndarray
-            Mode covariances.
+            State covariances.
         update_initializer : bool
             Do we want to use the passed means when we re-initialize
             the model?
@@ -593,17 +617,22 @@ class Model(ModelBase):
         dynemo_obs.set_means(self.model, means, update_initializer)
 
     def set_covariances(self, covariances, update_initializer=True):
-        """Set the covariances of each mode.
+        """Set the covariances of each state.
 
         Parameters
         ----------
         covariances : np.ndarray
-            Mode covariances.
+            State covariances.
         update_initializer : bool
             Do we want to use the passed covariances when we re-initialize
             the model?
         """
-        dynemo_obs.set_covariances(self.model, covariances, update_initializer)
+        dynemo_obs.set_covariances(
+            self.model,
+            covariances,
+            self.config.diagonal_covariances,
+            update_initializer,
+        )
 
     def set_trans_prob(self, trans_prob):
         """Sets the transition probability matrix.
@@ -682,8 +711,10 @@ class Model(ModelBase):
         """Set the means and covariances regularizer based on the training data.
 
         A multivariate normal prior is applied to the mean vectors with mu = 0,
-        sigma=diag((range / 2)**2) and an inverse Wishart prior is applied to the
-        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(range).
+        sigma=diag((range / 2)**2). If config.diagonal_covariances is True, a log
+        normal prior is applied to the diagonal of the covariances matrices with mu=0,
+        sigma=sqrt(log(2 * (range))), otherwise an inverse Wishart prior is applied
+        to the covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(1 / range).
 
         Parameters
         ----------
@@ -696,7 +727,9 @@ class Model(ModelBase):
             dynemo_obs.set_means_regularizer(self.model, training_dataset)
 
         if self.config.learn_covariances:
-            dynemo_obs.set_covariances_regularizer(self.model, training_dataset)
+            dynemo_obs.set_covariances_regularizer(
+                self.model, training_dataset, self.config.diagonal_covariances
+            )
 
     def get_alpha(self, dataset, concatenate=False):
         """Get state probabilities.
@@ -724,7 +757,7 @@ class Model(ModelBase):
                 x = data["data"]
                 g, _ = self._get_state_probs(x)
                 gamma.append(g)
-            alpha.append(np.concatenate(gamma))
+            alpha.append(np.concatenate(gamma).astype(np.float32))
 
         if concatenate or len(alpha) == 1:
             alpha = np.concatenate(alpha)
@@ -802,21 +835,31 @@ def _model_structure(config):
     data, gamma = tf.split(inputs, [config.n_channels, config.n_states], axis=2)
 
     # Definition of layers
-    means_layer = MeanVectorsLayer(
+    means_layer = VectorsLayer(
         config.n_states,
         config.n_channels,
         config.learn_means,
         config.initial_means,
         name="means",
     )
-    covs_layer = CovarianceMatricesLayer(
-        config.n_states,
-        config.n_channels,
-        config.learn_covariances,
-        config.initial_covariances,
-        config.covariances_epsilon,
-        name="covs",
-    )
+    if config.diagonal_covariances:
+        covs_layer = DiagonalMatricesLayer(
+            config.n_states,
+            config.n_channels,
+            config.learn_covariances,
+            config.initial_covariances,
+            config.covariances_epsilon,
+            name="covs",
+        )
+    else:
+        covs_layer = CovarianceMatricesLayer(
+            config.n_states,
+            config.n_channels,
+            config.learn_covariances,
+            config.initial_covariances,
+            config.covariances_epsilon,
+            name="covs",
+        )
     ll_loss_layer = CategoricalLogLikelihoodLossLayer(
         config.n_states, config.covariances_epsilon, name="ll_loss"
     )
