@@ -7,7 +7,6 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-import tensorflow as tf
 from tensorflow.keras import layers, models, optimizers, utils
 from tqdm import trange
 
@@ -15,7 +14,7 @@ from osl_dynamics.models import mdynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
 from osl_dynamics.inference.layers import (
     InferenceRNNLayer,
-    MeanVectorsLayer,
+    VectorsLayer,
     DiagonalMatricesLayer,
     CorrelationMatricesLayer,
     MixVectorsLayer,
@@ -581,7 +580,7 @@ def _build_inference_model(config):
     #   and the observation model.
 
     # Definition of layers
-    means_layer = MeanVectorsLayer(
+    means_layer = VectorsLayer(
         config.n_modes,
         config.n_channels,
         config.learn_means,
