@@ -14,9 +14,8 @@ from osl_dynamics.models import dynemo_obs
 from osl_dynamics.models.mod_base import BaseModelConfig, ModelBase
 from osl_dynamics.inference.layers import (
     add_epsilon,
-    DummyLayer,
     LogLikelihoodLossLayer,
-    MeanVectorsLayer,
+    VectorsLayer,
     CovarianceMatricesLayer,
     SubjectDevEmbeddingLayer,
     SubjectMapLayer,
@@ -27,7 +26,7 @@ from osl_dynamics.inference.layers import (
     SampleNormalDistributionLayer,
     SubjectMapKLDivergenceLayer,
     KLLossLayer,
-    MLPLayer,
+    MultiLayerPerceptronLayer,
 )
 
 
@@ -309,7 +308,10 @@ class Model(ModelBase):
             the model?
         """
         dynemo_obs.set_covariances(
-            self.model, group_covariances, update_initializer, layer_name="group_covs"
+            model=self.model,
+            covariances=group_covariances,
+            update_initializer=update_initializer,
+            layer_name="group_covs",
         )
 
 
@@ -335,7 +337,7 @@ def _model_structure(config):
         config.n_subjects, config.subject_embedding_dim, name="subject_embeddings"
     )
 
-    group_means_layer = MeanVectorsLayer(
+    group_means_layer = VectorsLayer(
         config.n_modes,
         config.n_channels,
         config.learn_means,
@@ -374,7 +376,7 @@ def _model_structure(config):
     )
 
     # Inference part of the deviation
-    means_dev_inf_input_layer = MLPLayer(
+    means_dev_inf_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -396,7 +398,7 @@ def _model_structure(config):
             name="means_dev",
         )
 
-    covs_dev_inf_input_layer = MLPLayer(
+    covs_dev_inf_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -476,7 +478,7 @@ def _model_structure(config):
     ll_loss = ll_loss_layer([data, m, C])
 
     # Prior part of deviations
-    means_dev_mod_input_layer = MLPLayer(
+    means_dev_mod_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -487,7 +489,7 @@ def _model_structure(config):
     means_dev_mod_sigma_layer = layers.Dense(
         config.n_channels, activation="softplus", name="means_dev_mod_sigma"
     )
-    covs_dev_mod_input_layer = MLPLayer(
+    covs_dev_mod_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -547,6 +549,7 @@ def get_group_means_covariances(model):
     group_covs = add_epsilon(
         group_covs_layer.bijector(group_covs_layer.flattened_cholesky_factors),
         group_covs_layer.epsilon,
+        diag=True,
     )
     return group_means.numpy(), group_covs.numpy()
 
